/************************************************************
 *
 * GUI Controller
 *
 ************************************************************/

/* 
 * A single controller for the whole GUI 
*/
Ext.define('POSync.controller.Gui', {
    extend: 'Ext.app.Controller',
    is: 'Gui',

    refs: [
        { ref: 'panel', selector: '#panel'},
        { ref: 'configPanel', selector: '#panel #config'},
        { ref: 'projects', selector: '#panel #projects'},
        { ref: 'files', selector: '#panel #files'},
        { ref: 'operations', selector: '#panel #operations'},
        { ref: 'projectSelector', selector: '#panel #files combobox'},
        { ref: 'localDir', selector: '#panel #localdir'},
        { ref: 'filterSelector', selector: '#panel #syncdir combobox'}
    ],

    /************************************************************
     *
     * Initialization
     *
     ************************************************************/

    /* Default values for the configuration data */
    configDefaults: {
        url:        'http://demo.project-open.net',
        email:      'ben.bigboss@tigerpond.com',
        password:   'ben',
        topdir:     './topdir',
    },

    /* Not really used right now, but canonical values of data
     * should be placed somewhere.
     */
    configData: {
        changed: false,
        url: null,
        email: null,
        password: null,
        topdir: null,
        form_is_valid: null,
        selected_project_id: null,
        token: null
    },

    /* This is called before the viewport is created */
    init: function() {
        console.log ('Gui.init: controller initialization');
        this.control({
            '#panel #config field': { change: this.onFieldChanged },
            '#panel toolbar > button': { click: this.onButtonClicked },
            '#panel #projects': { select: this.onProjectSelectedInProjects },
            '#panel #files combobox': { select: this.onProjectSelectedInFiles },
        });
    },

    /* This is called after the viewport is created */
    onLaunch: function() {
        console.log ('Gui.onLaunch: controller onLaunch');
 
        this.projectsStore = this.getProjects().getStore();
        this.filesStore = this.getFiles().getStore();
        this.localDirStore = this.getLocalDir().getStore();
	
        var app = this.getApplication();
        this.connectionController = app.getController('Connection');

        this.initConfiguration ();
        this.recoverState (); 
    },

    /* Initialize the configuration data */
    initConfiguration: function () {
        var configPanel = this.getConfigPanel();
        console.log('Gui.initConfiguration: config('+configPanel.id+')');

        const Store = require ('electron-store');
        const configStore = new Store ({name: 'config'});

        /* Initialize each configuration value to the saved one
         * or to the default value if none is already saved.
         */
        var defaults = this.configDefaults;
        for (var name in defaults) {
            var d = defaults[name];
            var v = configStore.get(name, d);
            var e = configPanel.down('[name='+name+']');
            e.setValue(v);
            this.configData[name] = v;
            /* Save the configuration value whenever it changes */
            e.on('change', function() {
                var name = this.name;
                var value = this.getValue();
		console.log ('Gui.initConfiguration.onChange('+name+'): changed to '+value+') - saving to configStore');
                configStore.set(name, value);
            });
        }
    },

    /* Recover the application state */
    recoverState: function () {
        var panel = this.getPanel();
        console.log('Gui.recoverState: panel('+panel.id+')');

        const Store = require ('electron-store');
        const stateStore = new Store ({name: 'state'});

        var w = stateStore.get('panel.width', 500);
        var h = stateStore.get('panel.height', 300);
        panel.setSize(w,h);
        panel.on ('resize', function() {
            var s = panel.getSize();
            stateStore.set({panel: s});
        });
    },

    /************************************************************
     *
     * Configuration data changes
     *
     ************************************************************/

    onFieldChanged: function (field, newValue, oldValue, eOpts) {
        var name = field.name;
        console.log ('Gui.onFieldChanged: field('+name+') changed from oldValue('+oldValue+') to newValue('+newValue+') eOpts('+eOpts+')');
        if (name in this.configDefaults && this.configData[name] != newValue) {
            console.log ('Gui.onFieldChanged: field('+name+') changed from('+this.configData[name]+') to('+newValue+')');
            this.configData[name] = newValue;
            this.configData.changed = true;
        }
    },

    /************************************************************
     *
     * Button actions
     *
     ************************************************************/

    onButtonClicked: function (button, event, eOpts) {
        console.log ('Gui.onButtonClicked: button clicked button('+button.getText()+') action('+button.action+') event('+event+') eOpts('+eOpts+')');
        switch (button.action) {
            case 'login':
                console.log('Gui.onButtonClicked: login clicked');
                if (this.formIsValid()) {
                        this.connectionController.login();
                } else {
                        console.log('Connection.login: form is not valid - cancelling request');
                        alert('Form is not valid');
                }
                break;
            case 'projects':
                console.log('Gui.onButtonClicked: projects clicked');
                this.connectionController.loadProjects();
                break;
            case 'files':
                console.log('Gui.onButtonClicked: files clicked');
                this.connectionController.loadFiles(45956);
                break;
            case 'localdir':
                console.log('Gui.onButtonClicked: localdir clicked');
                this.connectionController.loadLocalDirectory(45956);
                break;
            case 'sync':
                console.log('Gui.onButtonClicked: sync clicked');
                this.connectionController.sync(45956);
                break;
        }
    },

    /************************************************************
     *
     * Project selection
     *
     ************************************************************/

    /* Project selected by clicking a row in the projects panel */
    onProjectSelectedInProjects: function (row, record, index, eOpts) {
        var id = record.get('project_id');
        console.log ('Gui.onProjectSelectedInProjects: id('+id+')');
        this.selectProject (record);
    },

    /* Project selected in the selection box in the files panel */
    onProjectSelectedInFiles: function (combo, records, eOpts) {
        /* The combobox must be configured for 'single' selection mode */
        var id = records[0].get('project_id');
        console.log ('Gui.onProjectSelectedInFiles: id('+id+')');
        this.selectProject (records[0]);
    },

    /* Select the specified project and load its files */
    selectProject: function (project) {
        var id = project.get('project_id');
        console.log('Gui.selectProject (project) id('+id+')');
        console.log(project);
        if (this.configData.selected_project_id != id) {
            this.configData.selected_project_id = id;
            /* Update the selected row in the projects panel */
            this.getProjects().getSelectionModel().select (project);
            /* Update the selected project in the combobox of the files panel */
            this.getProjectSelector().select(project);
            /* Load the files of the selected project */
            var me = this;
            project.getFile(function(file,operation){
                me.selectedProjectFilesLoaded (project,file,operation);
            });
        }
    },

    /* Called whenever the list of files of the selected project have been received */
    selectedProjectFilesLoaded: function (project,file,operation) {
        var pid = project ? project.get('project_id') : 'undef';
        var fid = file ? file.get('id') : 'undef';
        console.log('Gui.selectedProjectFilesLoaded(project,file,operation): project_id('+pid+') file_id('+fid+')');
        console.log(project);
        console.log(file);
        console.log(operation);
        /* Show the files in the Files panel */
        this.getFiles().setRootNode(file);
        /* Update sync remote project files */
        this.syncRemoteFiles (project,file);
    },

    /************************************************************
     *
     * Filter selection
     *
     ************************************************************/

    /* Project selected in the selection box in the files panel */
    onFilterSelected: function (combo, records, eOpts) {
        /* The combobox must be configured for 'single' selection mode */
        var id = records[0].get('id');
        console.log ('Gui.onFilterSelected(combo,records,eOpts): id('+id+')');
        console.log(combo);
        console.log(records);
        console.log(eOpts);
        this.filterSyncNodes (id);
    },

    /* Select the specified project and load its files
     *
     * WARNING: esto ha sido un puto infierno. He probado MUCHAS formas
     * de hacerlo y todas fallaban por una u otra razón. Ésta parece que funciona.
     */
    filterSyncNodes: function (id) {
        console.log('Gui.filterSyncNodes: id('+id+')');
        var view = this.getSyncDir().getView();
        this.syncDirStore.forEachRecord (function (record) {
            var v = record.filteredBy (id);
            var el = Ext.get(view.getNode (record));
            /* el is defined only for currently expanded nodes */
            if (el)
                el.setDisplayed(v);
        });
    },

    /* Check if the form data is valid */
    formIsValid: function () {
        var config = this.getConfigPanel();

        this.configData.url = config.down('[name=url]').getValue();
        this.configData.email = config.down('[name=email]').getValue();
        this.configData.password = config.down('[name=password]').getValue();
        this.configData.topdir = config.down('[name=topdir]').getValue();
        this.configData.form_is_valid = config.getForm().isValid();

        var msg  = [
            'form ' + (this.configData.form_is_valid ? 'is' : 'is not') + ' valid and '
                + (this.configData.changed ? 'is' : 'is not') + ' changed',
            'URL     : "' + this.configData.url + '"',
            'Email   : "' + this.configData.email + '"',
            'Password: "' + this.configData.password + '"',
            'Topdir  : "' + this.configData.topdir + '"'
        ].join('\n');

        console.log (msg);
        return this.configData.form_is_valid;
    },

    /************************************************************
     *
     * Window management
     *
     ************************************************************/

    /* Hide the window but keep the tray icon */
    hideWindow: function () {
        console.log('Gui.hideWindow');
        const { remote } = require('electron');
        /* FIXME: should be the Gui window of this controller's application */
        remote.BrowserWindow.getFocusedWindow().hide();
    },

    /* Close the window and remove the tray icon */
    closeWindow: function () {
        console.log('Gui.closeWindow');
        const { remote } = require('electron');
        /* FIXME: should be the Gui window of this controller's application */
        remote.BrowserWindow.getFocusedWindow().close();
    },

});
